package cn.estsh.i3plus.core.apiservice.controller.base;

import cn.estsh.i3plus.core.api.iservice.base.ISysUserService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.core.api.iservice.busi.ICoreTreeService;
import cn.estsh.i3plus.core.apiservice.controller.DemoAuthController;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.Department;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;

/**
 * @Description : 用户权限服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 21:01
 * @Modify :
 **/
@RestController
@RequestMapping("/auth")
@Api(description="用户所有权限操作服务")
public class AuthController {

    public static final Logger LOGGER = LoggerFactory.getLogger(DemoAuthController.class);

    @Autowired
    private ISystemLoginService systemLoginService;

    @Autowired
    private ISysUserService userService;


    @GetMapping(value="/login")
    @ApiOperation(value="登陆",notes="登陆")
    public ResultBean login(HttpServletRequest request, String loginName, String loginPwd, String languageCode){
        LOGGER.info("用户登陆 loginName:{} loginPwd:{} languageCode:{}",loginName,loginPwd,languageCode);
        try {
            ValidatorBean.checkNotNull(loginName,"用户名不能为空");
            ValidatorBean.checkNotNull(loginPwd,"密码不能为空");
            ValidatorBean.checkNotNull(languageCode,"语言不能为空");

            SessionUser user = userService.queryUserLogin(loginName,loginPwd,languageCode);

            LOGGER.info("会员登陆：{}",user);
            AuthUtil.setSessionObject("languageCode",languageCode);
            String sessionId = request.getSession().getId();
            String token = "{accessToken:'"+sessionId+"',expiresIn:1800}";
            return new ResultBean(true,token,AuthUtil.getSessionUser());
        } catch (ImppBusiException e) {
            return new ResultBean(false,e.getErrorDetail() + ",so:" + e.getErrorSolution());
        }catch (Exception e){
            return ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    /**
     *
     * @return
     */
    @GetMapping(value="/logout")
    @ApiOperation(value="登出",notes="登出")
    public ResultBean logout(){
        LOGGER.info("用户退出...");
        try {
            AuthUtil.logout();
            return new ResultBean(true,"");
        } catch (ImppBusiException e) {
            return new ResultBean(false,e.getErrorDetail() + ",so:" + e.getErrorSolution());
        }
    }

    /**
     *
     * @return
     */
    @GetMapping(value="/getUserInfo")
    @ApiOperation(value="获取登录用户信息",notes="获取当前登录用户详细信息")
    public ResultBean getUserInfo(){
        LOGGER.info("获取用户信息...lang:{}",AuthUtil.getSessionAttribute("languageCode"));
        try {
            SessionUser sUser = AuthUtil.getSessionUser();
            LOGGER.info("用户：{}",sUser);
            return new ResultBean(true,"",sUser);
        } catch (ImppBusiException e) {
            return new ResultBean(false,e.getErrorDetail() + ",so:" + e.getErrorSolution());
        }
    }


}
