package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDictionaryService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.platform.bean.SysDictionary;
import cn.estsh.i3plus.pojo.platform.repository.SysDictionaryRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 字典服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-28 23:44
 * @Modify :
 **/
@Service
public class SysDictionaryServiceImpl implements ISysDictionaryService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysDictionaryServiceImpl.class);

    @Autowired
    private SysDictionaryRepository sysDictionaryRDao;

    @Override
    public void insertSysDictionary(SysDictionary sysDictionary) {
        // 查询父级字典名称
        if (sysDictionary.getParentId() != null && sysDictionary.getParentId() > 0) {
            LOGGER.info("字典 SYS_DICTIONARY parentId:{}", sysDictionary.getParentId());
            SysDictionary parentSysDictionary = sysDictionaryRDao.getById(sysDictionary.getParentId());
            ValidatorBean.checkNotNull(parentSysDictionary);

            sysDictionary.setRedParentName(parentSysDictionary.getName());
            sysDictionary.setRedParentCode(parentSysDictionary.getDictionaryCode());
        }

        LOGGER.info("字典 SYS_DICTIONARY sysDictionary:{}",sysDictionary);
        sysDictionaryRDao.insert(sysDictionary);
    }

    @Override
    public void deleteSysDictionaryById(String id) {
        LOGGER.info("字典 SYS_DICTIONARY id:{}",id);
        sysDictionaryRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void updateSysDictionary(SysDictionary sysDictionary) {
        // 查询父级字典名称
        if (sysDictionary.getParentId() != null && sysDictionary.getParentId() > 0) {
            LOGGER.info("字典 SYS_DICTIONARY parentId:{}", sysDictionary.getParentId());
            SysDictionary parentSysDictionary = sysDictionaryRDao.getById(sysDictionary.getParentId());
            ValidatorBean.checkNotNull(parentSysDictionary);

            sysDictionary.setRedParentName(parentSysDictionary.getName());
            sysDictionary.setRedParentCode(parentSysDictionary.getDictionaryCode());
        }

        LOGGER.info("字典 SYS_DICTIONARY sysDictionary:{}",sysDictionary);
        sysDictionaryRDao.update(sysDictionary);
    }

    @Override
    public List<SysDictionary> listSysDictionary() {
        LOGGER.info("字典 SYS_DICTIONARY list");
        return sysDictionaryRDao.list();
    }

    @Override
    public SysDictionary getSysDictionaryById(String id) {
        LOGGER.info("字典 SYS_DICTIONARY id:{}",id);
        return  sysDictionaryRDao.getById(Long.parseLong(id));
    }

    @Override
    public List<SysDictionary> findSysDictionaryByParentCode(String parentCode) {
        LOGGER.info("字典 SYS_DICTIONARY parentCode:{}",parentCode);
        return sysDictionaryRDao.findByProperty("redParentCode",parentCode);
    }

    @Override
    public ListPager querySysDictionaryByPager(SysDictionary sysDictionary, Pager pager) {
        LOGGER.info("字典 SYS_DICTIONARY SysDictionary:{}，Pager:{}",sysDictionary,pager);
        if(sysDictionary == null) {
            pager = PagerHelper.getPager(pager, sysDictionaryRDao.listCount());
            return new ListPager(sysDictionaryRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysDictionary(sysDictionary);
            pager = PagerHelper.getPager(pager, sysDictionaryRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysDictionaryRDao.findByHqlWherePage(hqlPack + sysDictionary.orderBy(),pager),pager);
        }
    }
}
