package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
import cn.estsh.i3plus.pojo.platform.repository.SysMenuRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 系统功能业务接口实现 
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 15:18:14.969
 * @Modify      :
 **/
@Service
public class SysMenuServiceImpl implements ISysMenuService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysMenuServiceImpl.class);

    @Autowired
    private SysMenuRepository sysMenuRDao;

    @Override
    public void updateSysMenu(SysMenu sysMenu) {
        LOGGER.info("系统功能 SYS_MENU :{}", sysMenu);
        sysMenuRDao.update(sysMenu);
    }

    @Override
    public void updateSysMenuStatus(String id, int status, SessionUser user) {
        LOGGER.info("系统功能 SYS_MENU  id:{} status:{} user:{}", id,status,user);
        SysMenu menu = sysMenuRDao.getById(Long.parseLong(id));
        menu.setMenuStatus(status);
        menu.setModifyUser(user.getUserName());
        sysMenuRDao.update(menu);
    }

    @Override
    public void deleteSysMenuById(String id) {
        LOGGER.info("系统功能 SYS_MENU Key:{}", id);
        long count = sysMenuRDao.findByPropertyCount("parentId",Long.parseLong(id));
        if(count >= 1){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode())
                    .setErrorDetail("该数据有子集")
                    .setErrorSolution("请先删除子集在操作")
                    .build();

        }
        sysMenuRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public SysMenu insertSysMenu(SysMenu sysMenu) {
        LOGGER.info("系统功能 SYS_MENU :{}", sysMenu);
        if(0 != sysMenu.getParentId()){
            SysMenu parent = sysMenuRDao.getById(sysMenu.getParentId());
            if(null != parent){
                sysMenu.setRedParentName(parent.getName());
            }

        }
        return sysMenuRDao.insert(sysMenu);
    }

    @Override
    public List<SysMenu> findSysMenuAll() {
        LOGGER.info("系统功能 SYS_MENU find All");
        return sysMenuRDao.findAll();
    }

    @Override
    public SysMenu getSysMenuById(String id) {
        LOGGER.info("系统功能 SYS_MENU find id:{}", id);
        return sysMenuRDao.getOne(Long.parseLong(id));
    }
}
