package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.base.ISysUserService;
import cn.estsh.i3plus.core.apiservice.dao.IUserPermissionDao;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.shirotoken.AdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.SaAdminToken;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.*;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.apache.shiro.authc.AuthenticationToken;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 平台用户业务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 17:45
 * @Modify :
 **/
@Service
public class SysUserServiceImpl implements ISysUserService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private SysUserRepository sysUserRDao;

    @Autowired
    private SysUserInfoRepository sysUserInfoRDao;

    @Autowired
    private IUserPermissionDao userPermissionDao;

    @Autowired
    private OrganizeRepository organizeRDao;

    @Autowired
    private SysRoleRepository sysRoleRDao;

    @Autowired
    private DepartmentRepository departmentRDao;

    @Autowired
    private PositionRepository positionRDao;

    @Autowired
    private RefUserRoleRepository refUserRoleRDao;

    @Autowired
    private RefUserDepartmentRepository refUserDepartmentRDao;

    @Autowired
    private RefUserPositionRepository refUserPositionRDao;

    @Override
    public SessionUser queryUserLogin(String loginName, String password, String languageCode) {
        LOGGER.info("平台用户 SYS_USER loginName:{} \t password:{} \t languageCode:{}", loginName, password, languageCode);

        SysUser user = getSysUserByLoginName(loginName);

        if(user == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.LOGIN_USER_NAME_EXCEPTION.getCode())
                    .setErrorDetail("用户名不存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }

        AuthenticationToken token = null;
        switch (user.getUserTypeId()) {
            case 2:
                token = new AdminToken(loginName, password, languageCode);
                break;
            case 3:
                token = new SaAdminToken(loginName, password, languageCode);
                break;
            default:
                token = new UserToken(loginName, password, languageCode);
                break;
        }
        SessionUser sessionUser = AuthUtil.login(token);

        user = dealSysUserInfo(user); //设置用户详细信息
        packSessionUser(sessionUser, user, user.getUserTypeId(), languageCode);
        AuthUtil.setSessionUser(sessionUser);
        return sessionUser;
    }


    @Override
    public void updateSysUserStatus(String id, int status, SessionUser user) {
        LOGGER.info("平台用户 SYS_USER id:{} status:{} modifyUser:{}", id, status, user.getUserName());
        SysUser sysUser = sysUserRDao.getById(Long.parseLong(id));
        sysUser.setUserStatusId(status);
        sysUser.setModifyUser(user.getUserName());
        sysUserRDao.update(sysUser);
    }

    @Override
    public void updateSysUser(SysUser sysUser) throws Exception{
        LOGGER.info("平台用户 SYS_USER SysUser :{}", sysUser);

        SysUser loginUser = getSysUserByLoginName(sysUser.getUserLoginName());
        // 判断是否修改登录名称
        if (loginUser == null || loginUser.getId().equals(sysUser.getId())) {
            // 密码加密 输入的密码不等于密码一致时修改密码
            if(!sysUser.getUserLoginPassword().isEmpty()){
                if(sysUser.getUserLoginPassword().equals(loginUser.getUserLoginPassword())){
                    String md5 = EncryptTool.hexMD5(sysUser.getUserLoginPassword());
                    sysUser.setUserLoginPassword(md5);
                }
            }
            sysUserInfoRDao.update(sysUser.getUserInfo());
            sysUserRDao.save(sysUser);
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("用户名已存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }
    }

    @Override
    public SysUser insertSysUser(SysUser sysUser )throws Exception {
        LOGGER.info("平台用户 SYS_USER SysUser:{}", sysUser);

        SysUser loginUser = getSysUserByLoginName(sysUser.getUserLoginName());
        if (null == loginUser) {
            // 密码加密 输入的密码不等于密码一致时修改密码
            String md5 = EncryptTool.hexMD5(sysUser.getUserLoginPassword());
            sysUser.setUserLoginPassword(md5);

            SysUser user = sysUserRDao.insert(sysUser);

            // 保存用户详细信息
            SysUserInfo userInfo = new SysUserInfo();
            userInfo.setUserId(user.getId());
            SysUserInfo info = sysUserInfoRDao.insert(userInfo);

            user.setUserInfo(info);
            return user;
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("用户名已存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }
    }

    @Override
    public void refreshRefUserDepartment(SysUser sysUser, String departmentId, SessionUser user) {
        LOGGER.info("平台用户 SYS_USER SysUser:{}", sysUser);
        LOGGER.info("平台用户 SYS_USER departmentId:{}", departmentId);
        LOGGER.info("平台用户 SYS_USER SessionUser:{}", user);

        refUserDepartmentRDao.deleteByProperties("userId", sysUser.getId()); //删除用户部门关系

        Department dep = departmentRDao.getById(Long.parseLong(departmentId));
        if(dep != null ){
            ValidatorBean.checkNotNull(dep);

            // 用户部门关系保存
            RefUserDepartment refUserDepartment = new RefUserDepartment(sysUser.getId(), dep.getId(), dep.getName());
            refUserDepartmentRDao.save(refUserDepartment);

            // 组织信息冗余
            Organize organize = organizeRDao.getById(dep.getOrganizeId());
            if(organize != null){
                sysUser.setRedOrganizeName(organize.getName());
                sysUser.setOrganizeId(organize.getId());
            }

            // 冗余 部门信息
            sysUser.setRedDepartmentName(dep.getName());
        }
    }

    @Override
    public void refreshRefUserRole(SysUser sysUser, String[] roleIds, SessionUser user) {
        LOGGER.info("平台用户 SYS_USER SysUser:{}", sysUser);
        LOGGER.info("平台用户 SYS_USER String[] :{}", roleIds);
        LOGGER.info("平台用户 SYS_USER SessionUser:{}", user);


        // 根据IDS 查询所有的角色信息
        String where = CoreHqlPack.packHqlIds("id", roleIds);
        List<SysRole> list = sysRoleRDao.findByHqlWhere(where);

        // 删除角色关系信息
        refUserRoleRDao.deleteByProperties("userId", sysUser.getId()); // 删除用户角色关系

        if(null != list && list.size() > 0){
            // 新增用户角色关系
            List<RefUserRole> refUserRoles = new ArrayList<>();
            List<String> roleNameList = new ArrayList<>();
            SysRole sysRole = list.get(0);
            RefUserRole refUserRole = null;
            for (SysRole role : list) {
                roleNameList.add(role.getName());
                // 用户角色关系创建
                refUserRole = new RefUserRole(sysUser.getId(), role.getId(), role.getName());
                refUserRoles.add(refUserRole);
            }
            refUserRoleRDao.saveAll(refUserRoles);

            // 角色名称冗余
            sysUser.getUserInfo().setRedRoleNames(String.join(",", roleNameList));
            sysUser.setRoleId(sysRole.getId());
            sysUser.setRedRoleName(sysRole.getName());
        }
    }

    @Override
    public void refreshRefUserPosition(SysUser sysUser, String positionId, SessionUser user) {
        LOGGER.info("平台用户 SYS_USER SysUser:{}", sysUser);
        LOGGER.info("平台用户 SYS_USER positionId :{}", positionId);
        LOGGER.info("平台用户 SYS_USER SessionUser:{}", user);

        Position pos = positionRDao.getById(Long.parseLong(positionId));
        ValidatorBean.checkNotNull(pos);

        refUserPositionRDao.deleteByProperties("userId", sysUser.getId()); //删除用户部门关系

        RefUserPosition refUserPosition = new RefUserPosition(sysUser.getId(), pos.getId(), pos.getName());
        refUserPositionRDao.insert(refUserPosition);

        // 岗位名称冗余
        sysUser.getUserInfo().setRedPositionNames(pos.getName());
        sysUser.setRedPositionName(pos.getName());
    }

    @Override
    public void deleteSysUserById(String id) {
        LOGGER.info("平台用户 SYS_USER DELETE By id :{}", id);
        sysUserRDao.updateByProperties("id", id, "isValid", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

        // 删除用户关系(物理删除)
        refUserRoleRDao.deleteByProperties("userId", Long.parseLong(id)); // 删除用户角色关系
        refUserDepartmentRDao.deleteByProperties("userId", Long.parseLong(id)); //删除用户部门关系
        refUserPositionRDao.deleteByProperties("userId", Long.parseLong(id)); //删除用户部门关系
    }

    @Override
    public List<SysUser> list() {
        LOGGER.info("平台用户 SYS_USER find All");
        return sysUserRDao.findByProperty("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
    }

    @Override
    public SysUser getSysUserById(String id) {
        LOGGER.info("平台用户 SYS_USER get By id :{}", id);
        return sysUserRDao.getById(Long.parseLong(id));
    }

    @Override
    public SysUserInfo getSysUserInfoById(String id) {
        LOGGER.info("平台用户明细 SYS_USER_INFO get By id :{}", id);
        return sysUserInfoRDao.getByProperty("userId",Long.parseLong(id));
    }

    @Override
    public SysUser getSysUserByLoginName(String loginName) {
        LOGGER.info("平台用户 SYS_USER find By Login Name :{}", loginName);
        return sysUserRDao.getByProperty(new String[]{"userLoginName", "isValid"},
                new Object[]{loginName, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()});
    }

    /**
     * 设置用户详细信息
     *
     * @param user
     * @return
     */
    @Override
    public SysUser dealSysUserInfo(SysUser user) {
        LOGGER.info("平台用户 SYS_USER user id :{}", user.getId());
        SysUserInfo userInfo = sysUserInfoRDao.getByProperty("userId", user.getId());
        Organize organize = organizeRDao.getById(user.getOrganizeId());

        List<Department> departmentList = userPermissionDao.findDepartmentByUser(user);
        List<Position> positionList = userPermissionDao.findPositionByUser(user);
        List<SysMenu> menuList = userPermissionDao.findSysMenuByUser(user);
        List<SysRole> roleList = userPermissionDao.findSysRoleByUser(user);

        LOGGER.info("平台用户 SYS_USER userInfo :{}", userInfo);
        LOGGER.info("平台用户 SYS_USER organize :{}", organize);
        LOGGER.info("平台用户 SYS_USER departmentList  size :{}", departmentList.size());
        LOGGER.info("平台用户 SYS_USER positionList size :{}", positionList.size());
        LOGGER.info("平台用户 SYS_USER menuList size :{}", menuList.size());
        LOGGER.info("平台用户 SYS_USER roleList size :{}", roleList.size());

        user.setUserInfo(userInfo);
        user.setOrganize(organize);
        user.setDepartmentList(departmentList);
        user.setMenuList(menuList);
        user.setPositionList(positionList);
        user.setRoleList(roleList);

        return user;
    }

    /**
     * 封装用户登陆信息
     *
     * @param sessionUser  当前会话信息
     * @param user         当前登录用户
     * @param userType     用户类型
     * @param languageCode 用户选择的语言
     * @return
     */
    public static SessionUser packSessionUser(SessionUser sessionUser, SysUser user, Integer userType, String languageCode) {
        sessionUser.setLanguageCode(languageCode);
        sessionUser.setUserName(user.getName());
        sessionUser.setUserTypeId(CommonEnumUtil.USER_TYPE.valueOf(userType));
        sessionUser.setUserCode(user.getUserInfo().getLanguageCode());
        sessionUser.setRoleList(user.getRoleList());
        sessionUser.setDepartmentList(user.getDepartmentList());
        sessionUser.setPositionList(user.getPositionList());
        sessionUser.setOrganize(user.getOrganize());

        if(null != user.getPositionList() && user.getPositionList().size() > 0){
            sessionUser.setPosition(user.getPositionList().get(0));
        }

        return sessionUser;
    }

}
